library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity zobrazovac is
	generic (number_of_segments : integer := 4);
	port (	CLK : in std_logic;
				DIGITS  : in std_logic_vector((number_of_segments * 4)-1 downto 0);
				POSITION : out std_logic_vector(number_of_segments-1 downto 0);
				DISPLAY  : out std_logic_vector(7 downto 0)
			);
end zobrazovac;

architecture Behavioral of zobrazovac is
	---
   component segment is
    Port ( DSELECT : in std_logic_vector(3 downto 0);
           DIGIT   : out std_logic_vector(7 downto 0)
          );
	end component;
   ---
   component clk_divisor is
   generic (divisor : natural );
   port (
      CLK_INPUT  : in std_logic;
      RESET      : in std_logic;
      CLK_OUTPUT : out std_logic
   );
   end component;
   -------------------------------------------------------------
	signal select_pos : std_logic_vector(1 downto 0);
	signal current_digit : std_logic_vector(3 downto 0);
   signal refresh : std_logic;
   ---
begin
	i_Clk_Divisor : clk_divisor
      generic map (divisor => 50_000 ) -- obnovovaci frekvence je 10 kHz
      port map (
         CLK_INPUT => CLK,
         RESET => '0',
         CLK_OUTPUT => refresh
      );
   
   -- dekoder 
   i_Segment : segment
		port map (	
						DSELECT => current_digit,
						DIGIT => DISPLAY -- vystup na piny sedmisegmentovky
					 );

	-- opakovany vyber pozice
	Obnovovani: 
	process (CLK)
		variable state : integer range 0 to 4;
	begin
		if CLK = '1' and CLK'EVENT then
		 	if refresh = '1' then
            state := state + 1;
            if state > 3 then state := 0; end if;
         end if;
		end if;		
		select_pos <= conv_std_logic_vector(state, select_pos'length);
	end process;

	POSITION <= not "0001" when select_pos = "00" else
			not "0010" when select_pos = "01" else
			not "0100" when select_pos = "10" else
			not "1000" when select_pos = "11";
	
	current_digit <= DIGITS(3 downto 0) when select_pos = "00" else
				  DIGITS(7 downto 4) when select_pos = "01" else
				  DIGITS(11 downto 8) when select_pos = "10" else
				  DIGITS(15 downto 12) when select_pos = "11";
end Behavioral;
